#define DEBUG 0
#include <Debug.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Node.h>
#include <fs_attr.h>

#include "BeFTPApp.h"
#include "BeFTPWindow.h"

BeFTPApp::BeFTPApp	(const char * a_app_sig)
 : BApplication	(a_app_sig)	
{
	argv_received	=	false;
	
	if (CreateTempFolder() != B_OK)
		exit(-1);	

	m_window = 	new BFWindow (BRect(300,200,495,320));
	
	GetPreferences();
}

BeFTPApp::~BeFTPApp	(void)	
{
	// void
}

void 
BeFTPApp::ArgvReceived	(int32 argc, char **argv)
{
	argv_received	=	true;

	if(argc <= 1)	// no arguments
		return;

	//	preprocess ftp arguments: host [port]

	for(int32 i = 1; i < argc; i++)
	{
		m_command = argv[i];
		m_command.ReplaceAll("ftp://", "");
		m_command.ReplaceAll(":", " ");
		
		int32 cursor = -1;
		if ((cursor = m_command.FindFirst("/")) >= 0)
			m_command.Truncate(cursor);
		
		m_command.Prepend("ftp ");
		
		PRINT(("\"%s\"", m_command.String()));
		
		RunScript();
	}
}

void
BeFTPApp::ReadyToRun	(void)
{
	if (!argv_received)
	{
		m_window->Show();
		return;
	}

	PostMessage(B_QUIT_REQUESTED);
}

void 
BeFTPApp::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what)
	{
		case FTP_HELP_REQUESTED:
				Help();
				break;
				
		default:
				BApplication::MessageReceived(a_message);
				break;
	}
}

void
BeFTPApp::AboutRequested	(void)
{
	BAlert * alert = new BAlert("BeFTP", "BeFTP 1.0\n\n"
	"Freeware   " B_UTF8_COPYRIGHT " Jonas Sundström, Kirilla.com\n", 
	"Help", "Kirilla", "Ok", B_WIDTH_AS_USUAL, B_INFO_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 1)
	{
		BMessage msg(B_ARGV_RECEIVED); 
		msg.AddString("argv", "app"); 
		msg.AddString("argv", "http://www.kirilla.com/");  
		msg.AddInt32("argc", 2); 
		be_roster->Launch ("application/x-vnd.Be.URL.http", &msg ); 			
	}
	
	if (reply == 0)
		Help();
}

void 
BeFTPApp::Help	(void)
{
	(new BAlert("BeFTP", 
	
	"BeFTP is a helper application that opens FTP URLs by "
	"opening a Terminal and running the 'ftp' utility.\n\n"

	"If you find 'ftp' lacking, remember that there are other clients "
	"avilable at www.bebits.com.\n\n"
	
	"The purpose of this application is mainly to show how to use the "
	"URL filetypes, and to supply the first(?) URL-savvy FTP client for BeOS.\n\n"

	"BeOS has a couple of URL filetypes, allowing "
	"applications to register that they can handle a certain URL. "
	"For example, an FTP client can specify that it handles FTP URL's. "
	"However, the URL isn't supposed to come to the application as a file, "
	"(there are no FTP, HTTP or Telnet files), "
	"but as an argument. "  
	"An application wanting to launch an FTP URL, but lacking the ability, "
	"can simply look at the FTP filetype, see which application is in charge of it, "
	"and open that application with the URL as the first and only argument. "
	"The shell utility 'ftp' doesn't do this by itself, "
	"so I wrote BeFTP to act as a proxy.\n\n"
	
	"Set BeFTP as the preferred application of the "
	"'application/x-vnd.Be.URL.ftp' filetype, in your "
	"FileTypes preferences.\n\n"
	
	"Currently, Opera and Mozilla do not support "
	"BeOS's URL mechanism. You will have to launch BeFTP separately. "
	"A workaround may be to copy the URL to your clipboard, from Opera/Mozilla, "
	"and use the Launch From Clipboard feature of Springboard, either from a script, "
	"from SpicyKeys, or from the Deskbar replicant.\n\n"
	
	"Btw, you can leave the 'Local dir' setting blank if you like. "
	"It will default to your home folder.", "Right", 
	NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT))->Go();
}

void 
BeFTPApp::ProcessRefs	(BMessage * a_message)
{

}

void 
BeFTPApp::GetPreferences(void)	
{
	// read preferences from app node

	// app ref
	app_info	my_app_info;
	be_app->GetAppInfo(& my_app_info);
	m_app_ref	=	my_app_info.ref;

	// app node
	BNode	node	(& m_app_ref);
	
	char *	buffer	=	new char [4096];
	off_t	bytes	=	0;

	// dir
	bytes = node.ReadAttr("beftp:dir", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		m_dir	=	buffer;
	}
	
	m_in_terminal	=	true;
	m_keep_open		=	false;
	
	delete [] buffer;
}

void 
BeFTPApp::RunScript(void)
{
	// create kickstart_script
	BString kickstart_script	("#!/bin/sh\n");

	// working dir
	kickstart_script.Append("cd ");	
	kickstart_script.Append(m_dir);	
	kickstart_script.Append("\n");	

	// command
	kickstart_script.Append(m_command);
	kickstart_script.Append("\n");
	
	// keep open
	if (m_keep_open && m_in_terminal)
		kickstart_script.Append("read -p \"\n(Press Enter to close)\"\n");
	
	//	create tempfile name
	int32	counter	=	1;
	BString tempfile_name;
	tempfile_name << counter;
	while(m_tmp_dir->Contains(tempfile_name.String()))
	{
		counter	++;
		tempfile_name.Truncate(0);
		tempfile_name << counter;
	}
	
	// write kickstart script to file
	BFile	tmp_file;
	if (m_tmp_dir->CreateFile(tempfile_name.String(), & tmp_file) != B_OK)
		exit(-1);
		
	tmp_file.Write(kickstart_script.String(), kickstart_script.Length());
	
	// open Terminal with kickstart script
	
	BPath	tempfile_path	(m_tmp_dir, tempfile_name.String());
	BString command_string	(tempfile_path.Path());

	if (m_in_terminal)
	{
		command_string.Prepend(" ");
		command_string.Prepend(m_app_ref.name);
		command_string.Prepend("/boot/beos/apps/Terminal -t ");
	}
	
	command_string.Append(" &");
	
	PRINT(("system():\n%s\n", command_string.String()));
	system(command_string.String());	
}

status_t
BeFTPApp::CreateTempFolder	(void)
{
	status_t status;
	BPath p;
	if (find_directory(B_COMMON_TEMP_DIRECTORY, &p) != B_OK)	return B_ERROR;
	
	m_tmp_dir	=	new BDirectory();
	BDirectory temp_dir;
	
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK) 	return status;
	status	=	m_tmp_dir->	CreateDirectory("Kirilla_BeFTP", &temp_dir);
	if (status != B_OK && status != B_FILE_EXISTS)	return status;

	p.SetTo(p.Path(),"Kirilla_BeFTP");
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK)	return status;
		
	BEntry entry(p.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}
